//********************************************************************
//  TicketCounter.java       Authors: Lewis/Chase
//
//  Demonstrates the use of a queue for simulating a waiting line.
//********************************************************************
import java.util.*;

public class TicketCounter    
{
   final static int PROCESS = 120;
   final static int MAX_CASHIERS = 10;
   final static int NUM_INTERVALS = 100;

   public static void main ( String[] args) 
   {
      int lambda;
      Customer customer;
      
      Scanner keyboard = new Scanner(System.in);
      System.out.println("Enter Lamda for this simulation: ");
      lambda = keyboard.nextInt();
      
      Queue<Customer> customerQueue = new LinkedList<Customer>();
      int[] cashierTime = new int[MAX_CASHIERS]; 
      int totalTime, averageTime, departs;

      /** process the simulation for various number of cashiers */
      for (int cashiers=0; cashiers < MAX_CASHIERS; cashiers++)
      { 
         /** set each cashiers time to zero initially*/
         for (int count=0; count < cashiers; count++)
            cashierTime[count] = 0;

         /** load customer queue - original text book code using fixed arrival times */
         //for (int count=1; count <= NUM_INTERVALS; count++)
         //   customerQueue.offer(new Customer(count*15));
         // same results each run: results =
         // cashiers   time      time (mins)
         //  1         5317              88
         //  2         2325              38
         //  3         1332              22
         //  4          840              14
         //  5          547               9
         //  6          355               6
         //  7          219               3
         //  8          120               2
         //  9          120               2
         // 10          120               2
         
         /** load customer queue - improved to use random Poisson arrival times*/
         Poisson myDist = new Poisson(lambda);
         for (int count=0; count < NUM_INTERVALS / lambda; count++) {
            int numberOfCustomers = myDist.getValue();
            for (int i = 0; i < numberOfCustomers; i++)
              customerQueue.offer(new Customer(count*15*lambda));
         }

         totalTime = 0;

         /** process all customers in the queue */
         while (!(customerQueue.isEmpty())) 
         {
            for (int count=0; count <= cashiers; count++)
            {
               if (!(customerQueue.isEmpty()))
               {
                  customer = customerQueue.poll();
                  if (customer.getArrivalTime() > cashierTime[count])
                     departs = customer.getArrivalTime() + PROCESS;
                  else
                     departs = cashierTime[count] + PROCESS; 
                  customer.setDepartureTime (departs);
                  cashierTime[count] = departs;
                  totalTime += customer.totalTime();
               }
            }
         }

         /** output results for this simulation */
         averageTime = totalTime / NUM_INTERVALS;
         System.out.println ("Number of cashiers: " + (cashiers+1));
         System.out.println ("Average time (mins): " + (int) (averageTime/60.0 + 0.5) + "\n");
      }
   }
}
